<?php

$shippingMethodsObj = new shippingMethods();

class shippingMethods {

    private $mDb;
    private $mConfig;
    private $dateTime;
    private $lang_code;

    public function __construct() {
        global $Config;
        $this->mDb = new iplus(); // تأكد من أن الكلاس iplus موجود ومُعرف بشكل صحيح
        $this->mConfig = $Config;
        $this->dateTime = date('Y-m-d H:i:s');
        $this->lang_code = LANGUAGE; // تأكد من أن ثابت LANGUAGE موجود
    }

    function getShippingMethods($aStart, $aLimit, $sort, $type, $searchName, $searchApiIdTo) {

        $sql = "SELECT p.* ,pl.name,pl.description,ia.name AS import_api_name , u.user_name AS 'addedBy'   FROM shipping_methods p";
        $sql .= " LEFT JOIN `shipping_methods_langs` pl ON p.id = pl.p_id ";
        $sql .= " LEFT JOIN `import_api` ia ON p.api_id_to = ia.id  and p.api_id_to !='0'";
        $sql .= " LEFT JOIN `users` u ON u.id = p.admin_id ";

        $sql .= " WHERE p.id > 0  AND pl.lang_code ='{$this->lang_code}'  ";
        $sql .= $searchName ? ' AND pl.name like "%' . $searchName . '%" ' : '';
        $sql .= $searchApiIdTo ? " AND p.api_id_to ='{$searchApiIdTo}'" : '';

        $sql .= " ORDER BY p.{$sort} {$type}";
        $sql .= $aLimit ? " LIMIT {$aStart}, {$aLimit}" : '';

        // echo $sql; die();

        return $this->mDb->getAll($sql);
    }

    function getSearchShippingMethodsCount($sort, $type, $searchName, $searchApiIdTo) {
        $sql = "SELECT COUNT(DISTINCT `id`) as 'result_count' FROM shipping_methods ";
        $sql .= " WHERE `id` > 0";
        $sql .= $searchName ? ' AND `name` like "%' . $searchName . '%"' : '';
        $sql .= $searchApiIdTo ? " AND  api_id_to ='{$searchApiIdTo}'" : '';
        $sql .= " ORDER BY {$sort} {$type}";

        //echo $sql; die();

        return $this->mDb->getOne($sql);
    }

    function getShippingMethodsCount() {
        try {
            return $this->mDb->connect()
                ->query("SELECT COUNT(*) FROM shipping_methods")
                ->fetchColumn() ?: 0;
        } catch (PDOException $e) {
            return 0;
        }
    }

    private function updateDefaultShippingMethod($currentId = 0) {
        $sql = "UPDATE `shipping_methods` SET `default_shipping_method` = :default_value WHERE `id` != :current_id";
        $stmt = $this->mDb->connect()->prepare($sql);
        return $stmt->execute(array(
            ':default_value' => '0',
            ':current_id' => $currentId
        ));
    }

    function addEditShippingMethods(array $temp) {
        // تحويل القيم المنطقية باستخدام Null coalescing operator (PHP 7+)
        $temp['active'] = !empty($temp['active']) ? '1' : '0';
        $temp['shipping_policy'] = !empty($temp['shipping_policy']) ? '1' : '0';
        $temp['default_shipping_method'] = !empty($temp['default_shipping_method']) ? '1' : '0';
        $temp['show_shipping_method_customer'] = !empty($temp['show_shipping_method_customer']) ? '1' : '0';

        $pdo = $this->mDb->connect();
        $pdo->beginTransaction();

        try {
            $isInsert = empty($temp['id']);
            $recordId = null;

            if ($isInsert) {
                $recordId = $this->insertShippingMethod($pdo, $temp);
                if ($recordId === false) {
                    throw new Exception("Failed to insert shipping method");
                }
            } else {
                $success = $this->updateShippingMethod($pdo, $temp);
                if ($success === false) {
                    throw new Exception("Failed to update shipping method");
                }
                $recordId = $temp['id'];
            }

            $pdo->commit();
            return $recordId;

        } catch (Exception $e) {
            $pdo->rollBack();
            return false;
        }
    }

    private function insertShippingMethod($pdo, $data) {
        try {
            $columns = $this->mDb->getCols('shipping_methods', $data);
            $insertData = array_filter(
                $columns,
                function($key) {
                    return !in_array($key, array('lang_Details', 'lang_parameter', 'governorateItem'));
                },
                ARRAY_FILTER_USE_KEY
            );

            if (empty($insertData)) {
                return false;
            }

            $insertColumns = array_keys($insertData);
            $insertValues = array_map(function($col) {
                return ":$col";
            }, $insertColumns);
            
            $sql = sprintf(
                "INSERT INTO `shipping_methods` (%s, `date_added`) VALUES (%s, :date_added)",
                implode(',', array_map(function($col) { return "`$col`"; }, $insertColumns)),
                implode(',', $insertValues)
            );

            $stmt = $pdo->prepare($sql);
            
            foreach ($insertData as $key => $value) {
                $stmt->bindValue(":$key", $value);
            }
            $stmt->bindValue(':date_added', $this->dateTime);

            if (!$stmt->execute()) {
                return false;
            }

            $id = $pdo->lastInsertId();
            if (empty($id)) {
                return false;
            }

            // إضافة تفاصيل اللغات
            if (!$this->insertLanguageDetails($pdo, $data['lang_Details'] ?? array(), $id)) {
                return false;
            }

            // إضافة تفاصيل المحافظات
            if (!$this->insertGovernorateDetails($pdo, $data['governorateItem'] ?? array(), $id)) {
                return false;
            }

            // تحديث الطريقة الافتراضية إذا لزم الأمر
            if ($data['default_shipping_method'] === '1') {
                $this->updateDefaultShippingMethod($id);
            }

            return $id;

        } catch (Exception $e) {
            return false;
        }
    }

    private function updateShippingMethod($pdo, $data) {
        try {
            if (empty($data['id'])) {
                return false;
            }

            $columns = $this->mDb->getCols('shipping_methods', $data);
            $updateData = array_filter(
                $columns,
                function($key) {
                    return !in_array($key, array('lang_Details', 'id', 'lang_parameter', 'governorateItem'));
                },
                ARRAY_FILTER_USE_KEY
            );

            if (empty($updateData)) {
                return false;
            }

            $updateParts = array_map(function($key) {
                return "`$key` = :$key";
            }, array_keys($updateData));
            
            $sql = sprintf(
                "UPDATE `shipping_methods` SET %s WHERE `id` = :id",
                implode(',', $updateParts)
            );

            $stmt = $pdo->prepare($sql);
            
            foreach ($updateData as $key => $value) {
                $stmt->bindValue(":$key", $value);
            }
            $stmt->bindValue(':id', $data['id']);

            if (!$stmt->execute()) {
                return false;
            }

            // حذف وإعادة إدخال تفاصيل اللغات
            if (!$this->deleteAndInsertLanguageDetails($pdo, $data)) {
                return false;
            }

            // حذف وإعادة إدخال تفاصيل المحافظات
            if (!$this->deleteAndInsertGovernorateDetails($pdo, $data)) {
                return false;
            }

            // تحديث الطريقة الافتراضية إذا لزم الأمر
            if ($data['default_shipping_method'] === '1') {
                $this->updateDefaultShippingMethod($data['id']);
            }

            return true;

        } catch (Exception $e) {
            return false;
        }
    }

    private function insertLanguageDetails(PDO $pdo, array $langDetails, $parentId): bool {
        foreach ($langDetails as $detail) {
            $sql = "INSERT INTO `shipping_methods_langs` 
                   (lang_code, name, description, p_id) 
                   VALUES (:lang_code, :name, :description, :p_id)";
            
            $stmt = $pdo->prepare($sql);
            if (!$stmt->execute(array(
                ':lang_code' => $detail['lang_code'],
                ':name' => $detail['details']['name'],
                ':description' => $detail['details']['description'],
                ':p_id' => $parentId
            ))) {
                return false;
            }
        }
        return true;
    }

    private function insertGovernorateDetails(PDO $pdo, array $governorateItems, $parentId): bool {
        foreach ($governorateItems as $item) {
            if (empty($item['governorate_id'])) {
                continue;
            }

            $sql = "INSERT INTO `shipping_methods_governorate` 
                   (governorate_id, store_price, delivery_time_from, 
                    delivery_time_to, branch_number, p_id, date_added) 
                   VALUES (:gov_id, :store_price, :time_from, 
                          :time_to, :branch_num, :p_id, :date_added)";
            
            $stmt = $pdo->prepare($sql);
            if (!$stmt->execute(array(
                ':gov_id' => $item['governorate_id'],
                ':store_price' => $item['store_price'],
                ':time_from' => $item['delivery_time_from'],
                ':time_to' => $item['delivery_time_to'],
                ':branch_num' => $item['branch_number'],
                ':p_id' => $parentId,
                ':date_added' => $this->dateTime
            ))) {
                return false;
            }

            $govId = $pdo->lastInsertId();
            if (!$this->insertCustomerPrices($pdo, $item['customer_price'] ?? array(), $govId)) {
                return false;
            }
        }
        return true;
    }

    private function insertCustomerPrices(PDO $pdo, array $prices, $govId): bool {
        foreach ($prices as $price) {
            $sql = "INSERT INTO `shipping_methods_governorate_customer_price` 
                   (p_id, affiliator_id, price, date_added) 
                   VALUES (:p_id, :aff_id, :price, :date_added)";
            
            $stmt = $pdo->prepare($sql);
            if (!$stmt->execute(array(
                ':p_id' => $govId,
                ':aff_id' => $price['affiliator_id'],
                ':price' => $price['price'] ?? 0,
                ':date_added' => $this->dateTime
            ))) {
                return false;
            }
        }
        return true;
    }

    private function deleteAndInsertLanguageDetails(PDO $pdo, array $data): bool {
        $stmt = $pdo->prepare("DELETE FROM `shipping_methods_langs` WHERE p_id = :p_id");
        if (!$stmt->execute(array(':p_id' => $data['id']))) {
            return false;
        }

        return $this->insertLanguageDetails($pdo, $data['lang_Details'] ?? array(), $data['id']);
    }

    private function deleteAndInsertGovernorateDetails(PDO $pdo, array $data): bool {
        $sql = "DELETE smg, amgcp FROM `shipping_methods_governorate` smg 
                LEFT JOIN `shipping_methods_governorate_customer_price` amgcp 
                ON (smg.id = amgcp.p_id) 
                WHERE smg.p_id = :p_id";
        
        $stmt = $pdo->prepare($sql);
        if (!$stmt->execute(array(':p_id' => $data['id']))) {
            return false;
        }

        return $this->insertGovernorateDetails($pdo, $data['governorateItem'] ?? array(), $data['id']);
    }

    function getOneShippingMethods($id) {
        $sql = "SELECT p.* FROM `shipping_methods` p ";
        $sql .= " WHERE p.`id` = '{$id}' ";
        $res = $this->mDb->getRow($sql);
        if ($res) {
            $sql = "SELECT lang_code ,name ,description FROM `shipping_methods_langs`";
            $sql .= " WHERE `p_id` = '{$id}' ";
            $res['lang_parameter'] = $this->mDb->getAll($sql);

            $sql = "SELECT id,governorate_id ,store_price,delivery_time_from,delivery_time_to,branch_number  FROM `shipping_methods_governorate`";
            $sql .= " WHERE `p_id` = '{$id}' ";
            $res['governorateItem'] = $this->mDb->getAll($sql);
            foreach ($res['governorateItem'] as $k => $v) {
                $sql = "SELECT affiliator_id ,price  FROM `shipping_methods_governorate_customer_price`";
                $sql .= " WHERE `p_id` = '{$v['id']}' ";
                $res['governorateItem'][$k]['customer_price'] = $this->mDb->getAll($sql);
            }
        }
        // echo "<pre>";print_r($res);die();
        return $res;
    }

    function getAllImportApiByActive() {
        $sql = "SELECT id,name FROM `import_api`  ";
        $sql .= " WHERE  `id` > '0'   ";
//        $sql .= " WHERE  `id` > '0' and active='1' ";
        $res = $this->mDb->getAll($sql);

        // echo "<pre>";print_r($res);die();
        return $res;
    }

    function deleteShippingMethods($ids) {
        $ids = implode("','", $ids);
        $imgListShipping_methods = $this->getAllImgShippingMethods($ids);
        $sql = "DELETE shipping_methods, shipping_methods_langs, shipping_methods_governorate, shipping_methods_governorate_langs,shipping_methods_governorate_customer_price
            FROM shipping_methods
            LEFT JOIN shipping_methods_langs ON shipping_methods.id = shipping_methods_langs.p_id
            LEFT JOIN shipping_methods_governorate ON shipping_methods.id = shipping_methods_governorate.p_id
            LEFT JOIN shipping_methods_governorate_langs ON shipping_methods_governorate.id = shipping_methods_governorate_langs.p_id
            LEFT JOIN shipping_methods_governorate_customer_price ON shipping_methods_governorate.id = shipping_methods_governorate_customer_price.p_id
            WHERE shipping_methods.id IN ('$ids')";

        $res = $this->mDb->query($sql);
        if ($res) {
            if (count($imgListShipping_methods) > 0) {
                foreach ($imgListShipping_methods as $k => $v) {
                    $old_img = $_SERVER['DOCUMENT_ROOT'] . '/' . $this->mConfig['uploads_path'] . DEALERUPLOAD . '/shipping-methods/' . $v['image'];
//                    echo $old_img."<hr>";
                    if (file_exists($old_img)) {
                        @unlink($old_img);
                    }
                }
            }
        }
        return $res;
    }

    public function getAllImgShippingMethods($ids) {
        $sql = "SELECT image  FROM `{$this->mPrefix}shipping_methods` ";
        $sql .= "WHERE id  in({$ids}) ";
//        echo $sql."<hr>";
        $res = $this->mDb->getAll($sql);

        return $res ? $res : array();
    }

//    function deleteShippingMethods($ids) {
//        foreach ($ids as $id) {
//            
//            $sql = "DELETE FROM `shipping_methods` ";
//            $sql.= " WHERE `id` = '{$id}'";
//            $res = $this->mDb->query($sql);
//            if ($res) {
//                //
//                $sql = "DELETE FROM `shipping_methods_langs` ";
//                $sql.= " WHERE `p_id` = '{$id}'";
//                $res1 = $this->mDb->query($sql);
//
//                //
//                $sql = "DELETE FROM `shipping_methods_governorate` ";
//                $sql.= " WHERE `p_id` = '{$id}'";
//                $res1 = $this->mDb->query($sql);
//                //
//                $sql = "DELETE FROM `shipping_methods_governorate_langs` ";
//                $sql.= " WHERE `p_id` = '{$id}'";
//                $res1 = $this->mDb->query($sql);
//            }
//        }
//
//        return $res;
//    }
}

?>